@@ -71,7 +71,7 @@ def get_source_target(source_file):
     source_target_edge_set.add( ( source, target ) )
 
   target_set = target_set - source_set
-  return  list( source_set ), list( target_set ), source_target_edge_set
+  return  list( source_set ), list( target_set ), list(source_target_edge_set)
 
 def generate_graph(tf_nodes, protein_nodes, edges):
   G = nx.DiGraph()
@@ -92,20 +92,34 @@ def filter_nodes(G, attrs_dic, over_condition=0):
   sub_G = G.subgraph(filter_nodes)
   return sub_G
 
-def network_summary_info(G, output_root):
+def network_summary_info(G, output_root, tf_num, enhancer_num):
   graph_info = {}
   graph_info['nodes_num'] = nx.number_of_nodes(G)
   graph_info['edges_num'] = nx.number_of_edges(G)
+  graph_info['tf_num'] = tf_num
+  graph_info['enhancer_num'] = enhancer_num
   graph_info['mean_degree'] = 2* graph_info['edges_num']/graph_info['nodes_num']
-  graph_info['assortativity_coefficient'] = nx.degree_assortativity_coefficient(G) # 匹配系数
-  graph_info['correlation_coefficient'] = nx.degree_pearson_correlation_coefficient(G) # 相关性系数
+  try:
+    graph_info['assortativity_coefficient'] = nx.degree_assortativity_coefficient(G) # 匹配系数
+  except Exception as e:
+    print( e )
+  try:
+    graph_info['correlation_coefficient'] = nx.degree_pearson_correlation_coefficient(G) # 相关性系数
+  except Exception as e:
+    print( e )
   # graph_info['closeness_centrality'] = nx.closeness_centrality(G) # 节点距离中心系数
   # graph_info['betweenness_centrality'] = nx.betweenness_centrality(G) # 节点介数中心系数
-  graph_info['transitivity'] = nx.transitivity(G) # 图或网络的传递性。
+  try:
+    graph_info['transitivity'] = nx.transitivity(G) # 图或网络的传递性。
+  except Exception as e:
+    print( e )
   # graph_info['clustering'] = nx.clustering(G) # 图或网络中节点的聚类系数,用来度量连接的密度。
   # graph_info['average_clustering'] = nx.average_clustering(G) # nx.clustering(G) 的均值
   # graph_info['square_clustering'] = nx.square_clustering(G)
-  graph_info['density'] = nx.density(G)
+  try:
+    graph_info['density'] = nx.density(G)
+  except Exception as e:
+    print( e )
   dic2csv(graph_info, output_root + 'csv/graph_info.csv')
 
 
@@ -182,6 +196,7 @@ def motif_analysis( G, output_root ):
     pickle.dump(motif_dic, f)
 
 
+
 def community_analysis( G, node_type_dic, output_root ):
   type_list = list( set( node_type_dic.values() ) )
   shape_list = 'od^ps>v<h8'
@@ -197,13 +212,13 @@ def community_analysis( G, node_type_dic, output_root ):
 
   dic2csv({'community_' + str(c): len(n_list) for c, n_list in community_dic.items()}, output_root + 'csv/community_node_num.csv')
   for c, n_list in community_dic.items():
+    with open( output_root + 'txt/community_%s.txt'%c, 'w' ) as f:
+      for n in n_list:
+        f.write(n + '\n')
+
     g = G.subgraph( n_list )
     keep_nodes = [n for n,d in g.degree if d >= 2 ]
     g = g.subgraph( keep_nodes )
-    with open( output_root + 'txt/community_%s.txt'%c, 'w' ) as f:
-      for n in g:
-        f.write(n + '\n')
-
     if len(g) <=10:
       print( 'community %s nodes less than 10'%str(c) )
       continue
@@ -240,63 +255,74 @@ def community_analysis( G, node_type_dic, output_root ):
     plt.savefig(output_root + "imgs/community_%s.png"%str(c), format="PNG")
     plt.close(fig)
 
+def main(tf_source=None, enhancer_source=None, select_nodes_file=None, express_file=None, output_root='./tmp_output/' ):
+  # 节点 + 边
+  total_source, total_target, total_edges =[], [], []
+  # 节点 属性
+  node_type_dic, node_express_dic = {}, {}
 
 
-def main(tf_source=None, tf_filter_nodes=None, enhancer_source=None, enhancer_filter_nodes=None, output_root='./tmp_output/' ):
-  filter_source, filter_target, edge_set =[], [], set()
-  node_type_dic = {}
-
-  # 1 使用source 构建网络
+  # 1 使用source file 获取 节点+边
+  tf_list, enhancer_list = [], []
   if tf_source:
-    tf_list, gene_list, tf_target_edge_set = get_source_target(tf_source)
+    tf_list, gene_list, tf_target_edge_list = get_source_target(tf_source)
     print( 'tf gene number is %s, target gene number is %s'%( str(len(tf_list) ), str(len(gene_list)) ))
-    if tf_filter_nodes:
-      filter_tf_nodes = [ln.strip() for ln in open(tf_filter_nodes)]
-      for tf in tf_list:
-        if tf in filter_tf_nodes:
-          filter_source.append( tf )
+    total_source.extend( list( set(tf_list) ) )
+    total_target.extend( list( set(gene_list) ) )
+    total_edges.extend( tf_target_edge_list )
 
-      for gene in gene_list:
-        if gene in filter_tf_nodes:
-          filter_target.append( gene )
-
-    else:
-      filter_source.extend(tf_list)
-      filter_target.extend(gene_list)
-    edge_set = tf_target_edge_set
+    # 节点属性
     for gene in gene_list:
       node_type_dic[gene] = 'gene'
     for tf in tf_list:
       node_type_dic[tf] = 'tf'
 
   if enhancer_source:
-    enhancer_list, gene_list, enhancer_gene_edge_set = get_source_target(enhancer_source)
+    enhancer_list, gene_list, enhancer_target_edge_list = get_source_target(enhancer_source)
     print( 'enhancer gene number is %s, target gene number is %s'%( str(len(enhancer_list) ), str(len(gene_list)) ))
-    if enhancer_filter_nodes:
-      filter_enhancer_nodes = [ln.strip() for ln in open(enhancer_filter_nodes)]
-      for enhancer in enhancer_list:
-        if enhancer in filter_enhancer_nodes:
-          filter_source.append( enhancer )
-      for gene in gene_list:
-        if gene in filter_enhancer_nodes:
-          filter_target.append( gene )
-    else:
-      filter_source.extend( enhancer_list  )
-      filter_target.extend( gene_list )
-
-    for edge in enhancer_gene_edge_set:
-      edge_set.add(edge)
+    total_source.extend( list( set(enhancer_list) ) )
+    total_target.extend( list( set(gene_list) ) )
+    total_edges.extend( enhancer_target_edge_list )
+    # 节点属性
     for gene in gene_list:
       if gene not in node_type_dic.keys():
         node_type_dic[gene] = 'gene'
     for enhancer in enhancer_list:
       node_type_dic[enhancer] = 'enhancer'
 
-  filter_source, filter_target = list(set(filter_source)), list(set(filter_target))
-  print( 'after filtering, source gene number is %s, target gene number is %s'%( str(len(filter_source) ), str(len(filter_target) )) )
 
-  G = generate_graph(filter_target, filter_source, edge_set)
-  network_summary_info(G, output_root)
+  # filter_nodes用于筛选的节点 包含了 tf, enhancer, gene。 最后的网络只在filter_nodes中
+  filtered_sources, filtered_targets = total_source, total_target
+  if select_nodes_file:
+    select_node_set = set([ln.strip() for ln in open(select_nodes_file)])
+    filtered_sources = list( select_node_set & set(filtered_sources) )
+    filtered_targets = list( select_node_set & set(filtered_targets) )
+
+  # print( len(filtered_targets), len(filtered_sources))
+  if express_file:
+    express_node_set = set()
+    for ln in open(express_file):
+      try:
+        gene, count = ln.strip().split(',')
+        if float(count) > 0:
+          express_node_set.add( gene )
+          node_express_dic[gene] = float(count)
+      except:
+        pass
+    # print( len(express_node_set) )
+    filtered_sources = list( set(filtered_sources) & express_node_set)
+    filtered_targets = list( set(filtered_targets) & express_node_set)
+
+  print( 'after filtering, source gene number is %s, target gene number is %s'%( str(len(filtered_sources) ), str(len(filtered_targets) )) )
+
+  filter_edges = []
+  for edge in total_edges:
+    s, t = edge[0], edge[1]
+    if s in filtered_sources and t in filtered_targets:
+      filter_edges.append(edge)
+
+  G = generate_graph(filtered_targets, filtered_sources, filter_edges)
+  network_summary_info(G, output_root, len( set(tf_list) & set(filtered_sources) ), len( set(enhancer_list) & set(filtered_sources) ) )
   # attr_dic = {ln.split(',')[0]: ln.split(',')[1] for ln in open(filter_nodes)}
   # print( len(protein_list), len(tf_list), len(tf_protein_edge_set))
   # G = generate_graph(protein_list, tf_list, tf_protein_edge_set)
